//*****************************************************************
//
// File Name	: 'procyon.c'
// Title		: Procyon MP3 player main
// Author		: Pascal Stang
// Date			: 11/22/2000
// Version		: 0.1
// Target MCU	: ATmega103
// Editor Tabs	: 3
//
//*****************************************************************


//#define AVR_AT90S8515
#define AVR_ATMEGA128

#ifndef WIN32
	#include <io.h>
	#include <signal.h>
	#include <interrupt.h>
#endif

#include "global.h"
#include "lcd.h"
#include "systimer.h"
#include "uart.h"
#include "i2c.h"
#include "spi.h"
#include "sta013.h"
#include "ata_if.h"
#include "playmgr.h"

//#include "procyon.h"

// function prototypes
void mainInitPorts(void);
void mainLCDtest(void);
void mainHDItest(void);
void mainSTA013test1(void);
void mainSTA013test2(void);
void mainLoadToBuffer(void);
void mainPlayFromBuffer(void);
void mainLoadToDisk(void);
void mainPlayFromDisk(void);
void mainPlayFromDisk2(void);
void mainDumpFromDisk(void);
void mainInitDecoder(void);



void mainInitPorts(void)
{
	// general configuration - safe all ports
	outp(0x00, DDRA);			// initialize PortA to input
	outp(0xFF, DDRB);			// initialize PortB to input
	//outp(0x00, DDRC);			// initialize PortC to input
	outp(0x00, DDRD);			// initialize PortD to input
	outp(0x00, DDRE);			// initialize PortE to input

	// specific configuration

	// STA013 data request
	cbi(STA013_DEMAND_PORT,STA013_DEMAND_PIN);	// set demand pin as input 

}

int main(void)
{
	unsigned int i;
	unsigned int j;
	unsigned char run;
	unsigned char s,r;

	//outp(0x7D, XDIV);			// set division ratio F_CPU/(129-d)
	//outp(0x7F, XDIV);			// set division ratio F_CPU/(129-d)
	//sbi(XDIV, XDIVEN);		// enable clock division

	mainInitPorts();				// initialize I/O ports
	timerInit0();				// initialize and start timebase
	uartInit();					// initialize uart
	i2c_init();					// initialize I2C interface
	spiInit();					// initialize spi interface
	ataInit();					// initialize ATA driver

	sbi(MCUCR, SRE);			// enable RAM interface

	timerPause(200);			// wait for hardware get up and running

	lcdInit();					// initialize LCD

	timerPause(200);			// wait for hardware get up and running

	lcdGotoXY(0,0); lcdPrintStr("--- Procyon MP3  ---");

	i = 0;
	j = 0;
	s = 0;
	r = 0;

	uartPrintStr("\r\n\n--- Procyon MP3  ---\r\n");
	//uartSendBuffer("Procyon MP3\n\r",13);
	uartPrintStr("Press ""?"" for help.\r\n");

	run = TRUE;

	while(run)
	{
		// print prompt
		uartPrintStr("\r\nProcyon MP3>");

		while(!uartReceiveByte(&s));
		uartPrintChar(s);
		uartPrintStr("\r\n\n");
		switch(s)
		{
		case '1':
			mainSTA013test1();
			break;
		case '2':
			mainPlayFromDisk2();
			break;
		case 'd':
			mainLCDtest();
			break;
		case 'h':
			mainHDItest();
			break;
		case 'i':
			mainInitDecoder();
			break;
		case 'k':
			mainLoadToBuffer();
			break;
		case 'l':
			mainLoadToDisk();
			break;
		case 'q':
			mainPlayFromBuffer();
			break;
		case 'p':
			mainPlayFromDisk();
			break;
		case 'u':
			mainDumpFromDisk();
			break;
		case '?':
			uartPrintStr("Commands:\r\n");
			uartPrintStr("(h) hard disk test\r\n");
			uartPrintStr("(d) lcd display test\r\n");
			uartPrintStr("(1) mp3 decoder test from serial port\r\n");
			uartPrintStr("(i) mp3 decoder initialize\r\n");
			uartPrintStr("(k) load data into buffer from serial port\r\n");
			uartPrintStr("(q) play mp3 from buffer\r\n");
			uartPrintStr("(l) load data into disk from serial port\r\n");
			uartPrintStr("(p) play mp3 from disk\r\n");
			uartPrintStr("(2) play mp3 from disk v2\r\n");
			uartPrintStr("(u) dump mp3 from disk to serial port (binary)\r\n");
			uartPrintStr("(?) for help\r\n");
			uartPrintStr("(e)xit\r\n");
			break;
		case 'e':
			uartPrintStr("Exiting...\r\n");
			run = FALSE;
			break;
		default:
			uartPrintStr("Unknown command\r\n");
			break;
		}
	}

	cbi(MCUCR, SRE);			// disable RAM interface

	return 0;
}

void mainLCDtest(void)
{
	u08 i,j;

	while(1)
	{
		lcdHome();
		for(j=0; j<9; j++)
		{
			timerPause(500);
			for(i=0; i<80; i++)
			{
				lcdDataWrite(0x30+((i+j)%10));
				outp(~i,PORTB);
			}
		}
	}
}

void mainSTA013test1(void)
{
	unsigned int i,j;
	unsigned char s;
	u32 bytesRead;

	i = 0;
	j = 0;

	lcdGotoXY(0,1); lcdPrintStr("Init STA013...      ");

	if(sta013Init())
	{
		lcdGotoXY(0,2); lcdPrintStr("No STA013           ");
		timerPause(2000);
	}
	else
	{
		lcdGotoXY(0,2); lcdPrintStr("STA013 OK          ");
		timerPause(2000);
	}
	sta013ResumeDecoder();

	//sta013HWReset();
	while(j != STA_IDENT)
	{
		j = sta013ReadReg(STA_REG_IDENT);
		lcdGotoXY(0,3); lcdPrintStr("STA013 ID: 0x"); lcdPrintfu08(j);
	}

	lcdGotoXY(0,2); lcdPrintStr("Bytes read:         ");

	cbi(DDRB, 7);	// set PB7 as input 

	bytesRead = 0;
	i = 0;

	while(1)
	{
		if(bit_is_set(PINB,PB7))
		{
			// data request high
			while(!uartReceiveByte(&s));
			spiSendByte(s);
			bytesRead++;
			lcdGotoXY(0,3);
			lcdPrintfu32(bytesRead);
			lcdPrintStr(" *");
			//lcdPrintfu32(TimeReg1);
			//lcdDataWrite(s);
			sbi(PORTB, PB6);
			//if(i)
			//{
			//	uartSendByte(1);
			//	i = 0;
			//}
		}
		else
		{
			// data request low
			lcdGotoXY(0,3);
			lcdPrintfu32(bytesRead);
			lcdPrintStr("  ");
			cbi(PORTB, PB6);
			//uartSendByte(2);
			//i = 1;
		}
	}

}

void mainInitDecoder(void)
{
	// initialize the mp3 decoder
	lcdGotoXY(0,1); lcdPrintStr("Init STA013...      ");
	uartPrintStr("Init STA013...\r\n");


	if(sta013Init())
	{
		lcdGotoXY(0,1); lcdPrintStr("No STA013           ");
		uartPrintStr("No STA013\r\n");
		timerPause(2000);
	}
	else
	{
		lcdGotoXY(0,1); lcdPrintStr("STA013 OK          ");
		uartPrintStr("STA013 OK\r\n");
		timerPause(2000);
	}
	sta013ResumeDecoder();
	uartPrintStr("Done!\r\n");
}

void mainLoadToBuffer(void)
{
	unsigned int i,j;
	unsigned char s;
	unsigned char *buffer;
	unsigned char *subbuffer;
	u32 LBAsector;
	u32 bytesRead;

	bytesRead = 0;

	i = 0;
	j = 0;

	// print intro message
	uartPrintStr("Load MP3 data into buffer via serial port\r\n");

	// setup our local buffer pointing to the big one
	buffer = (unsigned char*) MP3PLAY_BUFFER_ADDR;

	lcdGotoXY(0,1); lcdPrintStr("Bytes read:         ");
	bytesRead = 0;
	uartPrintStr("Ready for MP3 data...\r\n");

	// fill up the play buffer
	for(i=0; i < MP3PLAY_BUFFER_SIZE; i++)
	{
		// get data from serial port
		while(!uartReceiveByte(&s));
		// store to buffer
		buffer[i] = s;
		// do some stats
		bytesRead++;
		lcdGotoXY(0,2); lcdPrintfu32(bytesRead);
		//lcdGotoXY(0,3); lcdPrintfu32(j);
	}

	// store to disk just for fun
/*	uartPrintStr("Store to disk just for fun...\r\n");
	LBAsector = 0x00000001;
	uartPrintfu16((int) buffer);
	uartPrintStr("\r\n");
	for(j=0; j<(MP3PLAY_BUFFER_SIZE>>9); j++)
	{
		subbuffer = (buffer+(j*512));
		uartPrintStr("Storing sector");
		uartPrintfu16(j);
		uartPrintStr("\r\n");
		uartPrintfu16((int) subbuffer);
		uartPrintStr("\r\n");
		ataWriteSectorsLBA(0, LBAsector+j, 1, subbuffer );
	}

	//ataWriteSectorsLBA(0, LBAsector, 0x20, buffer );
*/
	uartPrintStr("\r\nDone Loading...\r\n");
	timerPause(4000);
	uartPrintStr("Flushing Receive Buffer\r\n");
	uartFlushReceiveBuffer();

}

void mainPlayFromBuffer(void)
{
	unsigned int i,j;

	i = 0;
	j = 0;
	
	// print intro message
	uartPrintStr("Play MP3 data from buffer\r\n");

	uartPrintStr("Initializing play manager\r\n");

	// initialize the play manager and buffers
	playmgrInit();
	// set play index to beginning of buffer
	PlayBuffer.dataindex = 0;

	// go to play
	lcdGotoXY(0,1); lcdPrintStr("Play:               ");

	uartPrintStr("Start Play...\r\n");
	playmgrStart();
	while(PlayBuffer.dataindex < PlayBuffer.size-0x80)
	{
		lcdGotoXY(0,2); lcdPrintfu32(PlayBuffer.dataindex);
		lcdGotoXY(0,3); lcdPrintfu32(j);
	}
	playmgrStop();
	uartPrintStr("Done!\r\n");
}

void mainLoadToDisk(void)
{
	unsigned int i,j;
	unsigned char s;
	unsigned char *buffer;
	u32 LBAsector;
	u32 bytesRead;

	i = 0;
	j = 0;

	// print intro message
	uartPrintStr("Load MP3 data into disk via serial port\r\n");

	// setup our local sector buffer
	buffer = (unsigned char*) SECTOR_BUFFER_ADDR;

	lcdGotoXY(0,1); lcdPrintStr("Bytes read:         ");
	bytesRead = 0;
	uartPrintStr("Ready for MP3 data...\r\n");

	// initialize LBA sector
	LBAsector = 0x00010000;

	// fill up 32 sectors
	for(j=0; j<0x040; j++)
	{
		// fill up a sector buffer
		for(i=0; i<0x200; i++)
		{
			// get data from serial port
			while(!uartReceiveByte(&s));
			// store to buffer
			buffer[i] = s;
			// do some stats
			bytesRead++;
			lcdGotoXY(0,2); lcdPrintfu32(bytesRead);
			//lcdGotoXY(0,3); lcdPrintfu32(j);
		}
		// write sector/buffer to disk
		//ataWriteSectorsLBAforCHS(0, LBAsector+j, 1, buffer);
		ataWriteSectorsLBA(0, LBAsector+j, 1, buffer);
	}

	uartPrintStr("\r\nDone Loading...\r\n");
	timerPause(4000);
	uartPrintStr("Flushing Receive Buffer\r\n");
	uartFlushReceiveBuffer();
}

void mainPlayFromDisk(void)
{

	unsigned int i,j;
	u08 burstSectors;
	u08* bufferptr;
	u32 LBAsector;

	i = 0;
	j = 0;

	// print intro message
	uartPrintStr("Play MP3 data from disk\r\n");

	uartPrintStr("Initializing play manager\r\n");
	// initialize the play manager and buffers
	playmgrInit();

	// initialize LBA sector
	//LBAsector = 0x00010000;
	LBAsector = 0x0000160F;
	//LBAsector = 0x00001890;

	// go to play
	lcdGotoXY(0,1); lcdPrintStr("Play:               ");
	lcdGotoXY(0,2); lcdPrintStr("Prebuffering...     ");

	// get the play buffer full
	// read sectors from disk into play buffer
	uartPrintStr("Read sectors from disk into buffer\r\n");
/*	for(j=0; j<PlayBuffer.size>>9; j++)
	{
		//ataReadSectorsLBAforCHS(0, LBAsector, 1, PlayBuffer.dataptr );
		ataReadSectorsLBA(0, LBAsector+j, 1, (PlayBuffer.dataptr+(j<<9)) );
		//timerPause(400);
	}
*/
	// read all sectors at once
	j = PlayBuffer.size>>9;
//	ataReadSectorsLBAforCHS(0, LBAsector, j, PlayBuffer.dataptr );
	ataReadSectorsLBA(0, LBAsector, j, PlayBuffer.dataptr );

	// set data length to reflect a full buffer
	PlayBuffer.datalength = PlayBuffer.size;

	ataPrintSector( PlayBuffer.dataptr );

	burstSectors = 1;

	uartPrintStr("Start Play...\r\n");
	playmgrStart();
//	while(j<0x60)
	while(1)
	{
		// print status
		lcdGotoXY(0,2); lcdPrintStr("P="); lcdPrintfu32(PlayBuffer.dataindex);
		lcdGotoXY(10,2); lcdPrintStr("  Len="); lcdPrintfu16(PlayBuffer.datalength);
		lcdGotoXY(0,3); lcdPrintStr("S="); lcdPrintfu32(j);

		if(PlayBuffer.datalength < (PlayBuffer.size-((burstSectors+1)*512)))
		{
			// buffer has one sector or more empty
			// time to fill up

			// get a pointer to the end of the buffer
			// ***should only be on sector boundaries
			bufferptr = PlayBuffer.dataptr + ((PlayBuffer.dataindex + PlayBuffer.datalength) & (PlayBuffer.size-1));
			// load in the new sectors
			for(i=0; i<burstSectors; i++)
			{
				ataReadSectorsLBA(0, LBAsector+j+i, 1, bufferptr+(i<<9));
			}
			// add sectors to the buffer length
			PlayBuffer.datalength += (burstSectors<<9);
			// increment LBA position
			j+=burstSectors;
		}

	}
	playmgrStop();
	uartPrintStr("Done!\r\n");

}

void mainPlayFromDisk2(void)
{

	unsigned int i,j,k;
	u08 burstSectors;
	u08* bufferptr;
	u32 LBAsector;

	i = 0;
	j = 0;

	// print intro message
	uartPrintStr("Play MP3 data from disk\r\n");

	uartPrintStr("Initializing play manager\r\n");
	// initialize the play manager and buffers
	playmgrInit();

	// initialize LBA sector
	//LBAsector = 0x00100000;
	//LBAsector = 0x0000160F;
	//LBAsector = 0x00001890;
	
	//LBAsector = 0x00000112;
	//LBAsector = 0x000002F0;
	LBAsector = 0x00000AA0;

	// go to play
	lcdGotoXY(0,1); lcdPrintStr("Play:               ");
	lcdGotoXY(0,2); lcdPrintStr("Prebuffering...     ");

	// get the play buffer full
	// read sectors from disk into play buffer
	uartPrintStr("Read sectors from disk into buffer\r\n");
	for(j=0; j<PlayBuffer.size>>9; j++)
	{
		ataReadSectorsLBAforCHS(0, LBAsector+j, 1, (PlayBuffer.dataptr+(j<<9)) );
		//ataReadSectorsLBA(0, LBAsector+j, 1, (PlayBuffer.dataptr+(j<<9)) );
		//timerPause(400);
	}

	// read all sectors at once
//	j = PlayBuffer.size>>9;
//	ataReadSectorsLBAforCHS(0, LBAsector, j, PlayBuffer.dataptr );
//	ataReadSectorsLBA(0, LBAsector, j, PlayBuffer.dataptr );

	// set data length to reflect a full buffer
	PlayBuffer.datalength = PlayBuffer.size;

//	ataPrintSector( PlayBuffer.dataptr );

	burstSectors = 1;
	k = (PlayBuffer.size>>9)-1;

	uartPrintStr("Start Play...\r\n");
	playmgrStart();
//	while(j<0x60)
	while(1)
	{
		// print status
		lcdGotoXY(0,2); lcdPrintStr("P="); lcdPrintfu32(PlayBuffer.dataindex);
		lcdGotoXY(10,2); lcdPrintStr("  Len="); lcdPrintfu16(PlayBuffer.datalength);
		lcdGotoXY(0,3); lcdPrintStr("S="); lcdPrintfu32(j);

		if(PlayBuffer.datalength < (PlayBuffer.size-((burstSectors)*512)))
		{
			// buffer has one sector or more empty
			// time to fill up

			// get a pointer to the end of the buffer
			// ***should only be on sector boundaries
			//bufferptr = PlayBuffer.dataptr + ((PlayBuffer.dataindex + PlayBuffer.datalength) & (PlayBuffer.size-1));
			// load in the new sectors
			for(i=0; i<burstSectors; i++)
			{
				//ataReadSectorsLBA(0, LBAsector+j+i, 1, PlayBuffer.dataptr+((j & k)<<9));
				ataReadSectorsLBAforCHS(0, LBAsector+j+i, 1, PlayBuffer.dataptr+((j & k)<<9));
			}
			// add sectors to the buffer length
			PlayBuffer.datalength += (burstSectors<<9);
			// increment LBA position
			j+=burstSectors;
		}

	}
	playmgrStop();
	uartPrintStr("Done!\r\n");

}

void mainDumpFromDisk(void)
{
	unsigned int i,j;
	unsigned char *buffer;
	unsigned char s;
	u32 LBAsector;
	u32 bytesSent;

	i = 0;
	j = 0;

	// print intro message
	uartPrintStr("Dump MP3 data from disk via serial port\r\n");

	// setup our local sector buffer
	buffer = (unsigned char*) SECTOR_BUFFER_ADDR;

	bytesSent = 0;
	uartPrintStr("Press key when ready for MP3 data...\r\n");
	while(!uartReceiveByte(&s));
	lcdGotoXY(0,1); lcdPrintStr("Sending...          ");

	// initialize LBA sector
	LBAsector = 0x0000160F;

	// fill up 0x80 sectors
	for(j=0; j<0x80; j++)
	{
		// fill up a sector buffer
		//cli();
		ataReadSectorsLBA(0, LBAsector+j, 1, buffer);
		//sei();
		// dump to serial port
		for(i=0; i<0x200; i++)
		{
			// send byte to serial port
			uartSendByte(buffer[i]);
			// do some stats
			bytesSent++;
			// print status
			lcdGotoXY(0,2); lcdPrintStr("B="); lcdPrintfu32(bytesSent);
			lcdGotoXY(0,3); lcdPrintStr("S="); lcdPrintfu32(j);
		}
	}

	uartPrintStr("\r\nDone Sending...\r\n");
	timerPause(4000);
	uartPrintStr("Flushing Receive Buffer\r\n");
	uartFlushReceiveBuffer();

}

void mainSTA013test2(void)
{

}


void mainHDItest(void)
{
	unsigned int i;
	unsigned int j;
	unsigned char run;
	unsigned char s,r;
	unsigned char head, sector;
	unsigned int track;
	unsigned long LBAsector;

	unsigned char* buffer = (unsigned char*) SECTOR_BUFFER_ADDR;

	i = 0;
	j = 0;
	s = 0;
	r = 0;

	head = 1;
	track = 0x10;
	sector = 1;
	LBAsector = 1;

	uartPrintStr("\r\n\nWelcome to Hard Disk Test.\r\n");
	uartPrintStr("Press ""?"" for help.\r\n");

	run = TRUE;

	while(run)
	{
		// print prompt & current CHS position
		uartPrintStr("\r\nHDtest");
		uartPrintStr(" H="); uartPrintfu08(head);
		uartPrintStr(" T="); uartPrintfu16(track);
		uartPrintStr(" S="); uartPrintfu08(sector);
		uartPrintStr(" LBA="); uartPrintfu32(LBAsector);
		uartPrintStr(">");

		while(!uartReceiveByte(&s));
		uartPrintChar(s);
		uartPrintStr("\r\n\n");
		switch(s)
		{
		case '1': head--;	break;
		case '2': head++; break;
		case '3': track--; break;
		case '4': track++; break;
		case '5': sector--; break;
		case '6': sector++; break;
		case '7': LBAsector--; break;
		case '8': LBAsector++; break;
		case '9': LBAsector-=0x100; break;
		case '0': LBAsector+=0x100; break;
		case 'r': 
			// read current sector to current buffer
			uartPrintStr("Reading H="); uartPrintfu08(head);
			uartPrintStr(" T="); uartPrintfu16(track);
			uartPrintStr(" S="); uartPrintfu08(sector);
			uartPrintStr(" into buffer...\r\n");
			ataReadSectors(0, head, track, sector, 1, buffer);
			break;
		case 'w':
			// write current buffer to current sector
			uartPrintStr("Writing H="); uartPrintfu08(head);
			uartPrintStr(" T="); uartPrintfu16(track);
			uartPrintStr(" S="); uartPrintfu08(sector);
			uartPrintStr(" from buffer...\r\n");
			ataWriteSectors(0, head, track, sector, 1, buffer);
			break;
		case 'z': 
			// read current sector to current buffer using LBA
			uartPrintStr("Reading LBA="); uartPrintfu32(LBAsector);
			uartPrintStr(" into buffer...\r\n");
			ataReadSectorsLBA(0, LBAsector, 1, buffer);
			break;
		case 'x':
			// write current buffer to current sector using LBA
			uartPrintStr("Writing LBA="); uartPrintfu32(LBAsector);
			uartPrintStr(" from buffer...\r\n");
			ataWriteSectorsLBA(0, LBAsector, 1, buffer);
			break;
		case 'c': 
			// read current sector to current buffer using LBA
			uartPrintStr("Reading LBA for CHS="); uartPrintfu32(LBAsector);
			uartPrintStr(" into buffer...\r\n");
			ataReadSectorsLBAforCHS(0, LBAsector, 1, buffer);
			break;
		case 'v':
			// write current buffer to current sector using LBA
			uartPrintStr("Writing LBA for CHS="); uartPrintfu32(LBAsector);
			uartPrintStr(" from buffer...\r\n");
			ataWriteSectorsLBAforCHS(0, LBAsector, 1, buffer);
			break;
		case 's':
			// print ata registers
			uartPrintStr("ATA registers:\r\n");
			ataShowRegisters(0);
			break;
		case 'k':
			// test reading sectors quickly
			uartPrintStr("Test reading 16MB from hard disk (at current sector):\r\n");
			for(i=0; i<0x8000; i++)
			{
				ataReadSectors(0, head, track, sector, 1, buffer);
			}
			uartPrintStr("Done!\r\n");
			break;
		case 'l':
			// test writing sectors quickly
			uartPrintStr("Test writing 16MB from hard disk (at current sector):\r\n");
			for(i=0; i<0x8000; i++)
			{
				ataWriteSectors(0, head, track, sector, 1, buffer);
			}
			uartPrintStr("Done!\r\n");
			break;
		case 'n':
			// do read from ATA_REG_DATA low and high for ~30 seconds
			uartPrintStr("Test reading from ATA_REG_DATA L/H:\r\n");
			for(j=0; j<0x0030; j++)
			{
				for(i=0; i<0x8000; i++)
				{
					r = *((volatile unsigned char*) ATA_REG_BASE + ATA_REG_DATAL);
					r = *((volatile unsigned char*) ATA_REG_BASE + ATA_REG_DATAH);
				}
				uartPrintStr(".");
			}
			uartPrintStr("Done!\r\n");
			break;
		case 'm':
			// do write to ATA_REG_DATA low and high for ~30 seconds
			uartPrintStr("Test writing to ATA_REG_DATA H/L:\r\n");
			for(j=0; j<0x0030; j++)
			{
				for(i=0; i<0x8000; i++)
				{
					*((volatile unsigned char*) ATA_REG_BASE + ATA_REG_DATAH) = r;
					*((volatile unsigned char*) ATA_REG_BASE + ATA_REG_DATAL) = r;
				}
				uartPrintStr(".");
			}
			uartPrintStr("Done!\r\n");
			break;
		case 'p':
			// print sector buffer
			ataPrintSector(buffer);
			break;
		case 'i':
			uartPrintStr("Reading drive identity into buffer...");
			// Issue identify command
  			ataWriteByte(ATA_REG_CMDSTATUS1, 0xEC);
			// wait for drive to request data transfer
			ataStatusWait(ATA_SR_DRQ, ATA_SR_DRQ);
			timerPause(200);
			// read in the data
			ataReadDataBuffer(buffer, 512);
			break;
		case '?':
			uartPrintStr("Commands:\r\n");
			uartPrintStr("(r)ead/(w)rite sector, (k/l) read/write 16MB, (n/m) read/write reg\r\n");
			uartPrintStr("(z) LBA read, (x) LBA write, (c) LBA readCHS, (v) LBA writeCHS\r\n");
			uartPrintStr("(p)rint buffer, (i)dentify drive, (s)how ATA registers\r\n");
			uartPrintStr("(1/2) head-/+, (3/4) track-/+, (5/6) sector-/+, (7/8) LBA-/+, (9,0) LBA -/+ 0x100\r\n");
			uartPrintStr("(?) for help, (e)xit\r\n");
			break;
		case 'e':
			uartPrintStr("Exiting Hard Disk Test...\r\n");
			run = FALSE;
			break;
		default:
			uartPrintStr("Unknown command\r\n");
			break;
		}
	}
}